<?php
namespace plugins\polls\controllers;

use plugins\polls\models\Poll;
use plugins\polls\models\VoteOption;
use plugins\polls\presenters\PollPresenter;
/**
 * Poll controller
 * @author Jérôme Brilland
 * @version 1.0.1
 * @copyright (c) 2022-2023, Jérôme Brilland
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License
 */
class PollController extends \CRUDController {
    
    protected $modelClassName = 'plugins\\polls\\models\\Poll';
        
    public function doList() {
        if ( 0 == Poll::count() ) {
            $this->redirect( BASE_URL . 'polls/poll/add' );
        }
        parent::doList();
        $authorization = \IoC::resolve( 'authorization' );
        $authorization->authorize( 'polls.view_polls' );
        $i18n = \IoC::resolve( 'i18n' );
        $this->title = $i18n->__( 'polls' );
        $this->polls = $this->models;
    }
    
    public function doAddform() {
        parent::doAddform();
        $authorization = \IoC::resolve( 'authorization' );
        $authorization->authorize( 'polls.create_poll' );
        $i18n = \IoC::resolve( 'i18n' );
        $this->title = $i18n->__( 'polls.add_poll' );
        if ( !count( $this->model->voteOptions ) ) {
            foreach ( range( 0, 2 ) as $i ) {
                $this->model->addVoteOption( new VoteOption() );
            }
        }
    }

    public function doAdd() {
        $authorization = \IoC::resolve( 'authorization' );
        $authorization->authorize( 'polls.create_poll' );
        parent::doAdd();
    }

    public function doUpdateform() {
        parent::doUpdateform();
        $authorization = \IoC::resolve( 'authorization' );
        $authorization->authorize( 'polls.update_poll' );
        $i18n = \IoC::resolve( 'i18n' );
        $this->title = $i18n->__( 'polls.update_poll' );
    }

    public function doUpdate() {
        $authorization = \IoC::resolve( 'authorization' );
        $authorization->authorize( 'polls.update_poll' );
        parent::doUpdate();
    }
    
    protected function beforeAdd() {
        foreach ( $this->model->voteOptions as $voteOption ) {
            if ( 0 == strlen( $voteOption->title ) ) {
                $voteOption->_delete = true;
            }
        }
    }
    
    protected function beforeUpdate() {
        foreach ( $this->model->voteOptions as $voteOption ) {
            if ( 0 == strlen( $voteOption->title ) ) {
                $voteOption->_delete = true;
            }
        }
    }

    protected function afterAdd() {
        $i18n = \IoC::resolve( 'i18n' );
        $this->flash['success'] = $i18n->__( 'polls.poll_successfully_created' );
        $this->redirect( BASE_URL . 'polls/poll/list' );
    }

    protected function afterUpdate() {
        $i18n = \IoC::resolve( 'i18n' );
        $this->flash['success'] = $i18n->__( 'polls.poll_successfully_updated' );
        $this->redirect( BASE_URL . 'polls/poll/list' );
    }

    public function doDelete() {
        $authorization = \IoC::resolve( 'authorization' );
        $authorization->authorize( 'polls.delete_poll' );
        parent::doDelete();
    }

    protected function afterDelete() {
        $i18n = \IoC::resolve( 'i18n' );
        $this->flash['success'] = $i18n->__( 'polls.poll_successfully_deleted' );
        $this->redirect( BASE_URL . 'polls/poll/list' );
    }
    
    public function doShow() {
        parent::doShow();
        $view = \IoC::resolve( 'view' );
        $totalVotes = function() {
            static $totalVotes = null;
            if ( is_null( $totalVotes ) ) {
                $totalVotes = 0;
                foreach( $this->poll->voteOptions as $voteOption ) {
                    $totalVotes += $voteOption->votes;
                }
            }
            return $totalVotes;
        };
        $view->getLatte()->addFunction( 'totalVotes', $totalVotes );
        $view->getLatte()->addFunction( 'percentage', function( $value ) use ( $totalVotes ) {
            return ( $totalVotes() > 0 ) ? $value / $totalVotes() * 100 : 0;
        } );
        $this->canUpdate = \IoC::resolve( 'authorization' )->isAuthorized( 'polls.update_poll' );
        $i18n = \IoC::resolve( 'i18n' );
        $this->title = $i18n->__( 'polls.results' );
    }
    
}
