<?php
namespace plugins\polls\models;

/**
 * Poll model class
 * @author Jérôme Brilland
 * @version 1.1
 * @copyright (c) 2022, Jérôme Brilland
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License
 */
 class Poll extends \ActiveRecord {
     
    private $voteOptions = [];
     
    protected function initialize() {
        $this->attach( new \NullifyBlankObserver() );
        
        if ( !isset( $this->start_date ) ) {
            $this->start_date = date( 'Y-m-d' );
        }
        if ( !isset( $this->start_time ) ) {
            $this->start_time = date( 'H:i:s' );
        }
    }
     
    protected function validate() {
        $i18n = \IoC::resolve( 'i18n' );
        $this->validateNotEmpty( 'topic', $i18n->__( 'polls.topic_cannot_be_empty' ) );
        $this->validateDate( 'start_date', $i18n->__( 'polls.invalid_start_date' ) );
        $this->validateNotEmpty( 'start_date', $i18n->__( 'polls.start_date_is_required' ) );
        $this->validateTime( 'start_time', $i18n->__( 'polls.invalid_start_time' ) );
        $this->validateNotEmpty( 'start_time', $i18n->__( 'polls.start_time_is_required' ) );
        if ( !empty( $this->end_date ) ) {
            $this->validateNotEmpty( 'end_time', $i18n->__( 'polls.end_time_is_required' ) );
            
            $startDate = new \DateTimeImmutable( $this->start_date . ' ' . $this->start_time );
            $endDate = new \DateTimeImmutable( $this->end_date . ' ' . $this->end_time );
            if ( $endDate <= $startDate ) {
                $this->addError( 'end_date', $i18n->__( 'polls.end_after_start') );
            }
        }
        foreach ( range( 0, 1 ) as $i ) {
            if ( !\Validator::validateNotEmpty( $this->_voteOptions[$i]->title ) ) {
                $this->errors['voteoptions'][$i + 1] = $i18n->__( 'polls.at_least_two_options' );
            }
        }
    }
       
    protected function beforeInsert() {
        $this->user_id = \IoC::resolve( 'user' )->user_id;
    }
    
    protected function beforeSave() {
        $this->topic = trim( strip_tags( $this->topic ) );
    }
    
    protected function afterSave() {
        foreach ( $this->voteOptions as $voteOption ) {
            if ( !empty( $voteOption->_delete ) ) {
                if ( !$voteOption->isNewRecord() ) {
                    $voteOption->delete();
                }
            } else {
                $voteOption->poll_id = $this->poll_id;
                $voteOption->save();
            }
        }
    }
    
    protected function getVoteOptions() {
        return $this->voteOptions ?: $this->voteOptions = VoteOption::findMany( '*', ['poll_id' => $this->poll_id] );
    }
    
    protected function setVoteOptionsAttributes( $voteOptionsAttributes ) {
        foreach ( $voteOptionsAttributes as $attributes ) {
            $this->voteOptions[] = VoteOption::findOrNew( ['vote_option_id' => $attributes['vote_option_id'] ?? null, 'poll_id' => $this->poll_id], $attributes );
        }
    }
    
    public function addVoteOption( $voteOption ) {
        $this->voteOptions[] = $voteOption;
    }
    
    public function isOpen() {
        $now = new \DateTimeImmutable();
        $startDate = new \DateTimeImmutable( $this->start_date . ' ' . $this->start_time );
        $endDate = ( !empty( $this->end_date ) ) ? new \DateTimeImmutable( $this->end_date . ' ' . $this->end_time ) : null;
        return ( $startDate <= $now ) && ( empty( $endDate ) || ( $endDate > $now ) );
    }
    
    public static function findOpenPolls() {
        $columns = [
            'poll_id',
            'topic',
            'language',
            'user_id',
            'allow_multiple_choice',
            'start_date',
            'start_time',
            'end_date',
            'end_time'
        ];
        $where = ['OR #first' => ['start_date[<]' => date( 'Y-m-d' ),
                                  'AND' => ['start_date' => date( 'Y-m-d' ), 'start_time[<=]' => date( 'H:i:s' )]],
                  'OR #second' => ['end_date' => null,
                                   'end_date[>]' => date( 'Y-m-d' ),
                                   'AND' => ['end_date' => date( 'Y-m-d' ), 'end_time[>]' => date( 'H:i:s' )]],
                  'ORDER' => ['start_date', 'start_time']];
        return Poll::findMany( $columns, $where );
    }
    
 }
